@@ -1,13 +1,11 @@
 ################################################################################
-### Part of the surveillance package, http://surveillance.r-forge.r-project.org
-### Free software under the terms of the GNU General Public License, version 2,
-### a copy of which is available at http://www.r-project.org/Licenses/.
-###
 ### Snapshot map (spplot) of an sts-object or matrix of counts
 ###
 ### Copyright (C) 2013-2014,2016,2017,2020,2021 Sebastian Meyer
-### $Revision: 2736 $
-### $Date: 2021-09-24 13:34:17 +0200 (Fri, 24. Sep 2021) $
+###
+### This file is part of the R package "surveillance",
+### free software under the terms of the GNU General Public License, version 2,
+### a copy of which is available at https://www.R-project.org/Licenses/.
 ################################################################################
 
 ## x: "sts" or (simulated) matrix of counts
@@ -29,6 +27,7 @@ stsplot_space <- function (x, tps = NULL, map = x@map, population = NULL,
                            total.args = NULL,
                            gpar.missing = list(col="darkgrey", lty=2, lwd=2),
                            sp.layout = NULL,
+                           ## aspect = mapasp(map), # currently hard-coded
                            xlim = bbox(map)[1, ], ylim = bbox(map)[2, ])
 {
     counts <- if (inherits(x, "sts")) observed(x) else x
@@ -72,7 +71,7 @@ stsplot_space <- function (x, tps = NULL, map = x@map, population = NULL,
         colorkey <- modifyList(eval(formals()$colorkey), colorkey)
 
     ## automatic additions to sp.layout (region labels and total)
-    if (is.list(gpar.missing) && any(is.na(map$ncases))) {
+    if (is.list(gpar.missing) && anyNA(map$ncases)) {
         layout.missing <- c(list("sp.polygons", obj=map[is.na(map$ncases),]),
                             gpar.missing)
         sp.layout <- c(sp.layout, list(layout.missing))
@@ -96,7 +95,8 @@ stsplot_space <- function (x, tps = NULL, map = x@map, population = NULL,
     ## generate the spplot()
     args <- list(quote(map[!is.na(map$ncases),]), "ncases", main=main,
                  col.regions=col.regions, at=at, colorkey=colorkey,
-                 sp.layout=sp.layout, xlim=xlim, ylim=ylim, quote(...))
+                 sp.layout=sp.layout, aspect=mapasp(map), xlim=xlim, ylim=ylim,
+                 quote(...))
     do.call("spplot", args)
 }
 
